from .actors import (
    Actor,
    DomainActor,
    WriteActor,
    ReadActor,
    AuditActor
)
from .messages import (
    Message,
    Command,
    Query,
    Event,
    Context,
    Error,
    IOEvent,
    BDEvent,
    StorageEvent,
    ResponseEvent,
    ReadEvent,
    WriteEvent,
    Frame,
    EventStream,
)



import abc
import typing
import collections.abc


class Message(abc.ABC):
    pass


class Context(abc.ABC):
    pass


Task = type('Task', (Message, abc.ABC), {})
Event = type('Event', (Message, abc.ABC), {})

Query = type('Query', (Task, abc.ABC), {})
Command = type('Command', (Task, abc.ABC), {})

Error = type('Error', (Event, abc.ABC), {})
BDEvent = type('BDE', (Event, abc.ABC), {})
IOEvent = type('IOE', (Event, abc.ABC), {})

StorageEvent = type('StorageEvent', (IOEvent, abc.ABC), {})
ReadEvent = type('ReadEvent', (StorageEvent, abc.ABC), {})
WriteEvent = type('WriteEvent', (StorageEvent, abc.ABC), {})

ResponseEvent = type('ResponseEvent', (IOEvent, abc.ABC), {})

P = typing.TypeVar('P', bound=Task)
C = typing.TypeVar('C', bound=Context)


class Frame(typing.Generic[P, C]):
    payload: P
    context: C


EventStream = collections.abc.Iterable[Event]



import collections.abc
from .messages import (
    Message,
    Command,
    Query,
    Error,
    BDEvent,
    IOEvent,
    Frame
)

Actor = collections.abc.Callable[[Message | Frame, ...], Message | collections.abc.Iterable[Message] | None]
DomainActor = collections.abc.Callable[[Command, ...], BDEvent | collections.abc.Iterable[BDEvent]]
WriteActor = collections.abc.Callable[[BDEvent, ...], IOEvent | collections.abc.Iterable[IOEvent] | Error | None]
ReadActor = collections.abc.Callable[[Query, ...], IOEvent | collections.abc.Iterable[IOEvent] | Error]
AuditActor = collections.abc.Callable[[Command | Query, ...], Command | Query | Error]



import typing
from .codex.query.bits import FilterBit, OrderBit, SliceBit, T


class EQ(FilterBit[T]):
    pass


class NE(FilterBit[T]):
    pass


class LE(FilterBit[T]):
    pass


class GE(FilterBit[T]):
    pass


class LT(FilterBit[T]):
    pass


class GT(FilterBit[T]):
    pass


class IS(FilterBit[T]):
    pass


class LIKE(FilterBit[T]):
    def __init__(self, value: T, case_sensitive: bool = False):
        super().__init__(value)
        self._case_sensitive = case_sensitive

    def __eq__(self, other: typing.Self):
        return (
            self._data == other.value and
            self._case_sensitive == other.case_sensitive
        )

    @property
    def case_sensitive(self) -> bool:
        return self._case_sensitive


class SET(FilterBit[T]):
    def __init__(self, *value: T):
        super().__init__(set(value))


class RANGE(FilterBit[T]):
    def __init__(self, left: GE | GT | None, right: LE | LT | None):
        super().__init__((left, right))


class NOT(FilterBit[T]):
    def __init__(self, value: FilterBit[T]):
        super().__init__(value)
        self._value = value

    def __eq__(self, other):
        if other is None:
            return super().__eq__(other)
        else:
            return super().__eq__(other.value)


class Limit(SliceBit):
    pass


class Offset(SliceBit):
    pass


class ASC(OrderBit):
    pass


class DESC(OrderBit):
    pass



import typing
import collections.abc
from .bits import ClauseBit

ParamName: typing.TypeAlias = str
ParamValue: typing.TypeAlias = str

NotationQuery: typing.TypeAlias = str | collections.abc.Mapping[ParamName, ParamValue]
NotationTypesMap: typing.TypeAlias = collections.abc.Mapping[ParamName, type]

NotationParser: typing.TypeAlias = typing.Callable[
    [NotationQuery, NotationTypesMap],
    collections.abc.Iterable[tuple[ParamName, ClauseBit]]
]
